#pragma once

/*
 * Ranges:
 * 0x0000 - 0x0FFF: Keystore Commands                    !!! Only add, never delete
 * 0x1000 - 0x1FFF: Protocol Commands                    !!! Only add, never delete
 * 0x2000 - 0x2FFF: Internal IPC Commands                !!! Add and delete allowed
 * 0x3000 - 0x3FFF: Configuration Parameter Commands     !!! Only add, never delete
 * 0x4000 - 0x4FFF: Internal Communication Marvell - DMU !!! Add and delete allowed
 * 0x5000 - 0x5FFF: Internal DMU commands                !!! Add and delete allowed
 */


namespace Hsm {

enum Command : uint16_t {
   CryptoCommand = 0x0000,
   CryptoProtocolCommand = 0x0000,
   CommandAsymmetricSignInit,                      //1
   CommandSignUpdate,                              //2
   CommandSignFinal,                               //3
   CommandAsymmetricSingleSign,                    //4
   CommandEncryptInit,                             //5
   CommandEncryptUpdate,                           //6
   CommandEncryptFinal,                            //7
   CommandSingleEncrypt,                           //8
   CommandCreateRsaKey,                            //9
   CommandCreateDsaKey,                            //10
   CommandCreateDhKey,                             //11
   CommandCreateAesKey,                            //12
   CommandCreateCamelliaKey,                       //13
   CommandExportRsaPublicKey,                      //14
   CommandStartKeyListing,                         //15
   CommandContinueKeyListing,                      //16
   CommandRenameKeys,                              //17
   CommandDeleteKeys,                              //18
   CommandStartCryptoLog,                          //19
   CommandContinueLog,                             //20
   CommandAddCertificate,                          //21
   CommandExportCertificate,                       //22
   CommandDeleteCertificate,                       //23
   CommandImportBlockCipherKey,                    //24
   CommandImportRsaPublicKey,                      //25
   CommandWrapKey,                                 //26
   CommandUnwrapKey,                               //27
   CommandDecryptInit,                             //28
   CommandDecryptUpdate,                           //29
   CommandDecryptFinal,                            //30
   CommandSingleDecrypt,                           //31
   CommandAsymmetricVerifyInit,                    //32
   CommandVerifyUpdate,                            //33
   CommandVerifyFinal,                             //34
   CommandAsymmetricSingleVerify,                  //35
   CommandTestKeyAccess,                           //36
   CommandImportRsaPrivateKey,                     //37
   CommandCreateDsaKeyParams,                      //38
   CommandCreateDsaKeyPair,                        //39
   CommandCreateDsaKeyParamsPair,                  //40
   CommandExportDsaParams,                         //41
   CommandExportDsaPublicKey,                      //42
   CommandImportDsaPublicKey,                      //43
   CommandImportDsaPrivateKey,                     //44
   CommandCreateDhKeyParams,                       //45
   CommandCreateDhKeyPair,                         //46
   CommandCreateDhKeyParamsPair,                   //47
   CommandExportDhParams,                          //48
   CommandExportDhPublicKey,                       //49
   CommandImportDhPublicKey,                       //50
   CommandImportDhPrivateKey,                      //51
   CommandExportBlockCipherKey,                    //52
   CommandAsymmetricDeriveKeyExport,               //53
   CommandAsymmetricDeriveKey,                     //54
   CommandGenerateRandom,                          //55 HMAC-DRBG
   CommandSeedRandom,                              //56
   CommandTestPerformance1,                        //57
   CommandTestPerformance2,                        //58
   CommandDigestInit,                              //59
   CommandDigestUpdate,                            //60
   CommandDigestFinal,                             //61
   CommandSingleDigest,                            //62
   CommandKeyInformation,                          //63
   CommandGetUserChallenge,                        //64
   CommandClearCryptoLog,                          //65
   CommandGetKeyFlag,                              //66
   CommandSetKeyFlag,                              //67
   CommandStartSyslogLog,                          //68
   CommandClearSyslogLog,                          //69
   CommandExportRsaPrivateKey,                     //70
   CommandExportDsaPrivateKey,                     //71
   CommandExportDhPrivateKey,                      //72
   CommandLoadPreloadKeys,                         //73
   CommandCreateTdeaKey,                           //74
   CommandSymmetricSignInit,                       //75
   CommandSymmetricSingleSign,                     //76
   CommandSymmetricVerifyInit,                     //77
   CommandSymmetricSingleVerify,                   //78
   CommandCreateEcKeyPair,                         //79
   CommandExportEcPublicKey,                       //80
   CommandExportEcPrivateKey,                      //81
   CommandImportEcPublicKey,                       //82
   CommandImportEcPrivateKey,                      //83
   CommandCreateEcParams,                          //84
   CommandSign,                                    //85
   CommandVerify,                                  //86
   CommandEncrypt,                                 //87
   CommandDecrypt,                                 //88
   CommandDigest,                                  //89
   CommandGetUserInfo,                             //90
   CommandCreateDhx942KeyParams,                   //91
   CommandCreateDhx942KeyPair,                     //92
   CommandCreateDhx942KeyParamsPair,               //93
   CommandExportDhx942Params,                      //94
   CommandExportDhx942PublicKey,                   //95
   CommandExportDhx942PrivateKey,                  //96
   CommandImportDhx942PublicKey,                   //97
   CommandImportDhx942PrivateKey,                  //98
   CommandGetKeySize,                              //99
   CommandGetKeyValue,                             //100
   CommandSetKeyValue,                             //101
   CommandCheckDrbgHmac,                           //102
   CommandGenerateEntropy,                         //103 Entropy after conditioning
   CommandSymmetricDeriveKeyExport,                //104
   CommandSymmetricDeriveKey,                      //105
   CommandCheckDrbgCtr,                            //106
   CommandCreateEd25519KeyPair,                    //107
   CommandExportEdPublicKey,                       //108
   CommandExportEdPrivateKey,                      //109
   CommandImportEd25519PublicKey,                  //110
   CommandImportEd25519PrivateKey,                 //111
   CommandAddDataObject,                           //112
   CommandExportDataObject,                        //113
   CommandDeleteDataObject,                        //114
   CommandExportSignedEkaKeyData,                  //115
   CommandEkaSign,                                 //116
   CommandCreateEkaTimeStamp,                      //117
   CommandBlockEkaKey,                             //118
   CommandUnblockEkaKey,                           //119
   CommandModifyEkaKeyAttributes,                  //120
   CommandCheckDsaPqGen,                           //121
   CommandCheckDsaGGen,                            //122
   CommandGenerateRandomDmu,                       //123
   CommandGenerateNoise,                           //124
   CommandRestartAndGenerateNoise,                 //125
   CommandImportRsaKeyPair,                        //126
   CommandImportDsaKeyPair,                        //127
   CommandImportDhKeyPair,                         //128
   CommandImportEcKeyPair,                         //129
   CommandImportDhx942KeyPair,                     //130
   CommandImportEd25519KeyPair,                    //131
   CommandCurrentlyUnused,                         //132
   CommandCreateCryptoCurrencyAddress,             //133
   CommandCreateIssKeyPair,                        //134
   CommandExportIssPublicKey,                      //135
   CommandExportIssPrivateKey,                     //136
   CommandImportIssPublicKey,                      //137
   CommandImportIssPrivateKey,                     //138
   CommandImportIssKeyPair,                        //139
   CommandDerivePrivateKey,                        //140
   CommandDerivePublicKey,                         //141
   CommandDeriveKeyPair,                           //142
   CommandExportEcParams,                          //143
   CommandEkaDecrypt,                              //144
   CommandContinueKeyListingNew,                   //145
   CommandDerivePublicKeyExport,                   //146
   CommandCreateCurve25519KeyPair,                 //147
   CommandImportCurve25519PublicKey,               //148
   CommandImportCurve25519PrivateKey,              //149
   CommandImportCurve25519KeyPair,                 //150
   CommandEkaUnwrap,                               //151
   CommandUserKeysStatistics,                      //152
   CommandAbortOperation,                          //153
   CommandImportPrivateKeyPkcs8,                   //154
   CommandImportKeyPairPkcs8,                      //155
   CommandCreateRsaAttestationKey,                 //156
   CommandCreateRsaTimestampKey,                   //157
   CommandExportSignedKeyData,                     //158
   CommandCreateKeyId,                             //159
   CommandDigestFinalSignTimestamp,                //160
   CommandSingleDigestSignTimestamp,               //161
   CommandSignTimestamp,                           //162
   CommandVerifyTimestamp,                         //163
   CommandCreateSymmetricKey,                      //164
   CommandCreatePasswordBasedKey,                  //165
   CommandExportPrivateKeyPkcs8Encrypted,          //166
   CommandExportPrivateKeyPkcs8Plain,              //167
   CommandVerifySignature,                         //168
   CommandCopyExternalObjectToInternalStorage,     //169
   CommandCopyInternalObjectToExternalStorage,     //170
   CommandCreateExternalRsaAttestationKey,         //171
   CommandCreateExternalRsaTimestampKey,           //172
   CommandGetUserCounterSerial,                    //173
   CommandGetUserCounterValue,                     //174
   CommandIncrementUserCounter,                    //175
   CommandCreateEcAttestationKey,                  //176
   CommandCreateEcTimestampKey,                    //177
   CommandCreateExternalEcAttestationKey,          //178
   CommandCreateExternalEcTimestampKey,            //179
   CommandEncodeSignature,                         //180
   LastCryptoCommand = CommandCreateExternalEcTimestampKey,
   CryptoHaCommand = 0x0400,
   CommandHaDeleteUser,                            //1
   CommandHaDeleteKeys,                            //2
   CommandHaDeleteKeysRequest,                     //3
   CommandHaDeleteCertificate,                     //4
   CommandHaDeleteCertificateRequest,              //5
   CommandHaRenameOld,                             //6
   CommandHaRenameRequest,                         //7
   CommandHaSetKeyFlagOld,                         //8
   CommandHaSetKeyFlagRequest,                     //9
   CommandHaAddClusterNode,                        //10
   CommandHaDeviceNameChange,                      //11
   CommandHaRenameUserOld,                         //12
   CommandHaUpdateUserPasswordOld,                 //13
   CommandHaUpdateUserSecretOld,                   //14
   CommandHaSetKeyValueOld,                        //15
   CommandHaSetKeyValueRequest,                    //16
   CommandHaUserSyncComplete,                      //17
   CommandHaDeleteDataObject,                      //18
   CommandHaDeleteDataObjectRequest,               //19
   CommandHaRenameUserNew,                         //20
   CommandHaBlockEkaKeyOld,                        //21
   CommandHaBlockEkaKeyRequest,                    //22
   CommandHaUnblockEkaKeyOld,                      //23
   CommandHaUnblockEkaKeyRequest,                  //24
   CommandHaModifyEkaKeyOld,                       //25
   CommandHaModifyEkaKeyRequest,                   //26
   CommandHaEkaSignCheck,                          //27
   CommandHaKeyStoreModification,                  //28
   CommandHaContinueUserSync,                      //29
   CommandHaRestoreKeys,                           //30
   CommandHaEkaDecryptCheck,                       //31
   CommandHaEkaUnwrapCheck,                        //32
   CryptoMgmtCommand = 0x0800,
   CommandMgmtNewSetupPassword,                    //1
   CommandMgmtNewPermanentSecret,                  //2
   CommandMgmtNewManagementPassword,               //3
   CommandMgmtGetUserConfig,                       //4
   CommandMgmtSetUserConfigValue,                  //5
   CommandMgmtUserKeysStatistics,                  //6
   CommandMgmtListInvalidatedKeys,                 //7
   CommandMgmtContinueKeyListing,                  //8
   CommandMgmtRecoverInvalidatedKey,               //9
   CommandMgmtDeleteInvalidatedKey,                //10
   CommandMgmtNewManagementSecret,                 //11
   CommandMgmtPersistNewManagementSecret,          //12
   CommandMgmtStartLogListing,                     //13
   CommandMgmtContinueLogListing,                  //14
   CommandMgmtAbortOperation,                      //15
   CommandMgmtContinueFileTransfer,                //16
   CommandMgmtApplyRestoreKeysList,                //17
   CommandMgmtApplyDeleteKeysList,                 //18
   CommandMgmtExportInvalidatedKeysList,           //19
   CommandMgmtImportInvalidatedKeysList,           //20
   CommandMgmtExportLogFile,                       //21
   CommandMgmtCreateBackup,                        //22
   CommandMgmtRestoreUserKeys,                     //23
   CommandMgmtTransmitBackup,                      //24
   CommandMgmtValidateUserBackup,                  //25
   CommandMgmtTransmitBackupPassword,              //26
   CommandMgmtExportShares,                        //27
   CommandMgmtRegisterShares,                      //28
   CommandMgmtAddShare,                            //29
   CommandMgmtDisplayShares,                       //30
   LastCryptoMgmtCommand = CommandMgmtDisplayShares,
   CryptoMgmtSetupCommand = 0x0900,
   CommandMgmtSetupExportSecret,                   //1
   CommandMgmtSetupFinalize,                       //2
   CommandMgmtSetupRequestShares,                  //3
   LastCryptoMgmtSetupCommand = CommandMgmtSetupRequestShares,
   ProtocolCommand = 0x1000,
   CommandLoginRequest,                            //1
   CommandLoginResponse,                           //2
   CommandGetDeviceInfo,                           //3
   CommandNoLongerUsed,                            //4
   CommandHello,                                   //5
   CommandGoodbye,                                 //6
   CommandStartEncryption,                         //7
   CommandPing,                                    //8
   CommandLoginChallenge,                          //9
   CommandCryptoLogin,                             //10
   CommandCryptoLogout,                            //11
   CommandListImplementedFunctions,                //12
   CommandLoginManagement,                         //13
   CommandLoginManagementSecret,                   //14
   CommandGetCurrentDeviceTime,                    //15
   LastProtocolCommand = CommandGetCurrentDeviceTime,
   HaCommand = 0x1100,
   CommandCompareConfiguration,                    //1
   CommandChangeConfigValue,                       //2
   CommandCompareCluster,                          //3
   CommandCompareUsers,                            //4
   CommandHaRequestUserData,                       //5
   CommandHaUserDataChunk,                         //6
   CommandHaUserDataComplete,                      //7
   CommandRequestObjectList,                       //8
   CommandObjectListChunk,                         //9
   CommandObjectListComplete,                      //10
   CommandObjectDataRequest,                       //11
   CommandObjectDataCompare,                       //12
   CommandRemoveNodeFromCluster,                   //13
   CommandCompareSoData,                           //14
   CommandSoDataUpdate,                            //15
   CommandMasterCardTypeUpdate,                    //16
   CommandHaConnectionStateUpdate,                 //17
   CommandHaConfigurationChanged,                  //18
   CommandUpdateRestoredUser,                      //19
   HaObjectStoreCommand = 0x1200,
   CommandNewObject,                               //1
   CommandUpdateObject,                            //2
   CommandRemoveObject,                            //3
   CommandNewObjectRejected,                       //4
   DecanusCommand = 0x1300,
   CommandDecanusAuthenticationRequest,            //1
   CommandExchangeSaltCommand,                     //2
   CommandDecanusChallengeRequest,                 //3
   FsmUserInterfaceCommand = 0x1400,
   CommandSendFsmSignal,                           //1 CommandSerialFsmSendSignal
   CommandLogoutNotification,                      //2 CommandSerialFsmRequestDoLogoutNotification
   CommandSendSoActivation,                        //3 CommandSerialFsmRequestDoSendSOActivation
   CommandAnnounceSoState,                         //4 CommandSerialFsmRequestDoAnnounceSoState
   DisplayCommand = 0x1500,
   CommandDisplayMessage,                          //1  CommandSerialFsmRequestDoSendToDisplay
   CommandDisplayEnterPin,                         //2  CommandSerialFsmRequestDoSendToDisplayEnterPin
   CommandDisplayReenterPin,                       //3  CommandSerialFsmRequestDoSendToDisplayReenterPin
   CommandDisplayEnterCard,                        //4  CommandSerialFsmRequestDoSendToDisplayEnterCard
   CommandDisplayAskUser,                          //5  CommandSerialFsmRequestDoAskUser
   CommandDisplayEnterUsername,                    //6  CommandSerialFsmRequestDoSendToDisplayEnterUsername
   CommandDisplayPassword,                         //7  CommandSerialFsmRequestDoSendPasswordToDisplay
   CommandDisplayEnterDeviceName,                  //8  CommandSerialFsmRequestDoSendToDisplayEnterDeviceName
   CommandDisplayEnterUID,                         //9  CommandSerialFsmRequestDoSendToDisplayEnterUID
   CommandDisplayEnterCardOwner,                   //10 CommandSerialFsmRequestDoSendToDisplayEnterCardOwner
   CommandDisplayFeatureFinished,                  //11 CommandSerialFsmRequestDoSendToDisplayFeatureFinished
   CommandDisplayDiagnostics,                      //12 CommandSerialFsmRequestDoSendToDisplayDiagnostics
   CommandDisplayEnterPassword,                    //13 CommandSerialFsmRequestDoSendToDisplayEnterPassword
   CommandDisplayFsmState,                         //14 CommandSerialFsmRequestDoShowLastState
   CommandDisplayEnterNewUsername,                 //15 CommandSerialFsmRequestDoSendToDisplayEnterNewUsername
   CommandDisplayEnterUsb,                         //16 CommandSerialFsmRequestDoSendToDisplayEnterUSB
   CommandDisplayDigitalSeal,                      //17 CommandSerialFsmRequestDoSendToDisplayDigitalSeal
   CommandDisplayChooseFile,                       //18
   CommandDisplayMessageConfirmed,                 //19
   CommandDisplayFeatureFinishedConfirmed,         //20
   CommandDisplayEnterPinNewCard,                  //21
   CommandDisplayEnterRep,                         //22
   CommandDisplayEnterRep2,                        //23
   CommandDisplayGenesisPin_Obsolete,              //24
   CommandDisplayEnterDisclosureSecret,            //25
   CardCommand = 0x1600,
   CommandCardNotifyStatus,                        //1  Send notifications about card events/status (insert/eject/...)
   CommandCardReadBlock,                           //2  Read a data block from card
   CommandCardWriteBlock,                          //3  Write a data block to card
   CommandCardVerifyPsc,                           //4  Verify PSC (programmable security code) on card
   FileCommand = 0x1700,
   CommandFileMonitorUsbStick,                     //1  CommandSerialFsmRequestDoSendMonitorUSBStick
   CommandFileStat,                                //2
   CommandFileReadRequest,                         //3
   CommandFileReadChunk,                           //4
   CommandFileWriteRequest,                        //5
   CommandFileWriteChunk,                          //6
   CommandFileDelete,                              //7
   CommandFileList,                                //8
   CommandFileAbort,                               //9
   CommandControllerCommunication = 0x1800,
   CommandControllerLogin,                         //1
   CommandControllerLogout,                        //2
   CommandControllerSetPin,                        //3
   CommandControllerSetUid,                        //4
   CommandControllerSetCardOwner,                  //5
   CommandControllerSetUsername,                   //6
   CommandControllerSetDevicename,                 //7
   CommandControllerLoginCheck,                    //8
   CommandControllerSetPassword,                   //9
   CommandControllerSignalNotAllowed,              //10
   CommandControllerRebootDevice,                  //11 CommandSerialFsmRequestDoReboot
   CommandControllerShutdownCpu,                   //12 CommandSerialFsmRequestDoShutdownCpu
   CommandControllerSnoozeBuzzer,                  //13
   CommandControllerGetDmuStatus,                  //14
   CommandControllerSetDisclosureSecret,           //15
   IpcCommand = 0x2000,
   IpcCryptoCommand = 0x2100,
   CommandUserPasswordLoginRequest,                //1
   CommandLoginLineEncryptionKey,                  //2
   CommandUserChallengeLoginRequest,               //3
   CommandChallengeLineEncryptionKey,              //4
   CommandCreateHaClientLogin,                     //5
   CommandCheckClientLoginAndCreateHaServerLogin,  //6
   CommandCheckServerLoginResponse,                //7
   CommandCreateCloneKeyMaterial,                  //8
   CommandSetKeyStoreKey,                          //9
   CommandSuspendCryptoInterface,                  //10
   CommandDecryptLocalCloneKeyAndSuspend,          //11
   CommandDecryptRemoteCloneKeyAndSuspend,         //12
   CommandCreateCloneKey,                          //13
   CommandDecryptCloneKey,                         //14
   CommandCreateBackupKeyMaterial,                 //15
   CommandResumeCryptoInterface,                   //16
   CommandCreateRemoteCloneKeys,                   //17
   CommandImportRemoteCloneInfo,                   //18
   CommandCheckUidInCluster,                       //19
   CommandSyncKeysToMaster,                        //20
   CommandGetClusterHash,                          //21
   CommandCompareClusterHash,                      //22
   CommandApplyMasterCluster,                      //23
   CommandNewHaUserNoLongerUsed,                   //24
   CommandAnnounceHaUser,                          //25
   CommandChangeDeviceName,                        //26
   CommandRemovedFromCluster,                      //27
   CommandLogAddNoLongerUsed,                      //28
   CommandLogHaRemoveNoLongerUsed,                 //29
   CommandCreateDecanusPairingMaterial,            //30
   CommandDecanusAuthentication,                   //31
   CommandDeleteUser,                              //32 CommandSerialFsmRequestDoDeleteUser
   CommandCreateUser,                              //33 CommandSerialFsmRequestDoCreateUser
   CommandGetCurrentNumberOfUsers,                 //34 CommandSerialFsmRequestDoGetNumberOfUsers
   CommandCreateNewUserSecret,                     //35 CommandSerialFsmRequestDoCreateNewSecret
   CommandCreateNewUserSetupPassword,              //36 CommandSerialFsmRequestDoGetPassword
   CommandRenameUser,                              //37 CommandSerialFsmRequestDoRenameUser
   CommandDeleteDecanusPairings,                   //38
   CommandLoginNewLineEncryptionKey,               //39
   CommandChallengeNewLineEncryptionKey,           //40
   CommandNewClientLoginAndCreateHaServerLogin,    //41
   CommandNewServerLoginResponse,                  //42
   CommandGenerateRandomInternal,                  //43
   CommandListInvalidatedObjects,                  //44
   CommandApplyInvalidatedObjectsList,             //45
   CommandDecanusChallenge,                        //46
   CommandRegisterInternalClient,                  //47
   CommandRestartPrngNoiseSource,                  //48
   CommandManagementPasswordLoginRequest,          //49
   CommandManagementLineEncryptionKey,             //50
   CommandCreateNewManagementPassword,             //51
   CommandManagementSecretLoginRequest,            //52
   CommandManagementSecretLineEncryptionKey,       //53
   CommandGetUserKek,                              //54
   CommandGetUserInfoInternal,                     //55
   CommandLoadRestoredUser,                        //56
   CommandSwitchUserId,                            //57
   CommandResetMsr,                                //58
   IpcNetworkCommand = 0x2200,
   CommandTcpDisconnectCleanup,                    //1
   CommandCloseAllOpenConnections,                 //2
   CommandCloseConnection,                         //3
   CommandRestartConnection,                       //4
   CommandPauseProtocolInterface,                  //5
   CommandResumeProtocolInterface,                 //6
   CommandMgmtTcpDisconnectCleanup,                //7
   IpcFsmCommand = 0x2300,
   CommandStartProcess,                            //1
   CommandStartNetwork,                            //2
   CommandStopNetwork,                             //3
   CommandGetIntegretyChecksum,                    //4
   CommandFactoryReset,                            //5
   CommandCheckUpdateFile,                         //6
   CommandUpdateSoftware,                          //7
   CommandRollbackSoftware,                        //8
   CommandCheckLicenseFile,                        //9
   CommandLicenseUpdate,                           //10
   CommandCheckDevicenameInCluster,                //11
   CommandRemoveDevicenameFromCluster,             //12
   CommandCheckDecanusInCluster,                   //13
   CommandRemoveDecanusFromCluster,                //14
   CommandSoDataChangeNotification,                //15
   CommandMasterSoDataChangeNotification,          //16
   CommandGetMasterCardType,                       //17
   CommandStartupSelftest,                         //18 CommandSerialFsmRequestDoSelftest
   CommandRestore,                                 //19 CommandSerialFsmRequestDoRestore
   CommandImportClone,                             //20 CommandSerialFsmRequestDoImportMasterData
   CommandCheckRestoreFile,                        //21 CommandSerialFsmRequestDoCheckRestoreKey
   CommandCheckCloneFile,                          //22 CommandSerialFsmRequestDoReadMasterData
   CommandCreateClone,                             //23 CommandSerialFsmRequestDoCreateCloneWriteOnStick
   CommandCreateCloneKeyStick,                     //24 CommandSerialFsmRequestDoCreateCTStick
   CommandCheckCloneKey,                           //25 CommandSerialFsmRequestDoGetCloneKeyFromStick
   CommandCreateBackup,                            //26 CommandSerialFsmRequestDoWriteBackupOnStick
   CommandExportLogs,                              //27 CommandSerialFsmRequestDoWriteLogsToStick
   CommandCreateHaCloneKeyStick,                   //28 CommandSerialFsmRequestDoCreateCTRemoteStick
   CommandImportHaCloneKeys,                       //29 CommandSerialFsmRequestDoGetCloneDataRemote
   CommandCreateHaClone,                           //30 CommandSerialFsmRequestDoCreateMasterStickRemote
   CommandCheckHaCloneFile,                        //31 CommandSerialFsmRequestDoReadMasterDataRemote
   CommandImportHaClone,                           //32 CommandSerialFsmRequestDoImportHaMasterData
   CommandStartSoTimeoutWatchdog,                  //33 CommandSerialFsmRequestDoStartSOTimeoutWatchdog
   CommandStopSoTimeoutWatchdog,                   //34 CommandSerialFsmRequestDoStopSOTimeoutWatchdog
   CommandCreateDecanusStick,                      //35 CommandSerialFsmRequestDoCreateDeviceDecanusStick
   CommandValidateBackup,                          //36 CommandSerialFsmRequestDoValidateBackup
   CommandGetCryptoMode,                           //37
   CommandSetCryptoMode,                           //38
   CommandPauseNetworkCryptoInterface,             //39
   CommandResumeNetworkCryptoInterface,            //40
   CommandCryptoLoginInternal,                     //41
   CommandExportInvalidatedObjectsList,            //42
   CommandImportRestoreList,                       //43
   CommandImportRemoveList,                        //44
   CommandCheckListFile,                           //45
   CommandResetSoTimeoutWatchdog,                  //46
   CommandSetupRks,                                //47
   CommandInstallRks,                              //48
   CommandCheckHaSetup,                            //49
   CommandCheckRksSetup,                           //50
   CommandCreateSystemKek,                         //51
   CommandRestoreUser,                             //52
   CommandRebootRequired,                          //53
   CommandCheckMgmtBackupKey,                      //54
   CommandExportMgmtBackupKey,                     //55
   CommandDecryptMgmtBackupKey,                    //56
   CommandCheckMgmtBackupFile,                     //57
   CommandImportUser,                              //58
   CommandCheckDeviceBackupFile,                   //59
   CommandCheckPartitionBackupFile,                //60
   IpcConfigCommand = 0x2400,
   CommandProcessConfig,                           //1
   CommandCryptoUserConfig,                        //2
   CommandDeleteUserConfig,                        //3
   CommandGetHaConfigHash,                         //4
   CommandCompareConfigHash,                       //5
   CommandApplyMasterHaConfig,                     //6
   CommandGetSerialNumber,                         //7
   CommandCloseUserConnections,                    //8
   CommandSetFilePeer,                             //9
   CommandCryptoStartupParameters,                 //10
   CommandGetLicenseCheckString,                   //11
   CommandGetPublicSerialNumber,                   //12
   CommandCloseManagementConnections,              //13
   CommandExportUserConfigInternal,                //14
   IpcEventCommand = 0x2500,
   EventHello,                                     //1
   EventIpcDisconnect,                             //2
   EventWatchdog,                                  //3
   EventFileChange,                                //4
   EventDnsChange,                                 //5
   EventFileReadComplete,                          //6
   EventFileWriteComplete,                         //7
   EventSystemState,                               //8
   EventSoLogin,                                   //9
   EventSoLogout,                                  //10
   EventConfirmedDisconnect,                       //11
   EventTcpDisconnect,                             //12
   IpcSoftHsmInterfaceMessages = 0x2600,
   CommandLoadAddedUser,                           //1
   DeviceConfigCommand = 0x3000,
   ReadConfigDeviceCommand = 0x3100,
   CommandSerialExportNetworkConfig,               //1
   CommandSerialGetSystemTime,                     //2
   CommandSerialGetDate_unsued,                    //3
   CommandSerialGetDeviceName,                     //4
   CommandSerialGetDNSIPv4,                        //5
   CommandSerialGetDNSIPv6,                        //6
   CommandSerialGetPort,                           //7
   CommandNoLongerUsedGetIPv6Port,                 //8
   CommandSerialGetDHCPIPv4,                       //9
   CommandSerialGetIPv6,                           //10
   CommandSerialGetIPv4,                           //11
   CommandSerialGetNmaskIPv4,                      //12
   CommandSerialGetGWIPv4,                         //13
   CommandSerialGetSyslogIP,                       //14
   CommandSerialGetCryptologIP,                    //15
   CommandSerialGetFsmState,                       //16
   CommandSerialGetSinglePort,                     //17
   CommandSerialGetNtpUrl,                         //18
   CommandSerialGetNtpKey,                         //19
   CommandSerialGetSyslogKey,                      //20
   CommandSerialGetCryptologKey,                   //21
   CommandSerialGetRedundancy,                     //22
   CommandSerialGetSyslogSize,                     //23
   CommandSerialGetCryptologSize,                  //24
   CommandSerialGetSyslogPort,                     //25
   CommandSerialGetCryptologPort,                  //26
   CommandSerialGetNmaskIPv6,                      //27
   CommandSerialGetGWIPv6,                         //28
   CommandSerialGetDNSDomain,                      //29
   CommandSerialGetDHCPIPv6,                       //30
   CommandSerialGetStaticNetworkConfig,            //31
   CommandSerialGetStaticIpv4RouteConfig,          //32
   CommandSerialGetStaticIpv6RouteConfig,          //33
   CommandSerialGetDecanus,                        //34
   CommandSerialGetTiltSensor,                     //35
   CommandSerialGetSyslogTcp,                      //36
   CommandSerialGetCryptologTcp,                   //37
   CommandSerialGetAutoNeg,                        //38
   CommandSerialGetLineSpeed,                      //39
   CommandSerialGetFullDuplex,                     //40
   CommandSerialGetProtcolInterface,               //41
   CommandSerialGetSnmpInterface,                  //42
   CommandSerialGetNtpInterface,                   //43
   CommandSerialGetSyslogInterface,                //44
   CommandSerialGetCryptologInterface,             //45
   ReadConfigSecurityCommand = 0x3200,
   CommandSerialExportSoConfig,                    //1
   CommandSerialGetNtp,                            //2
   CommandSerialGetBackupAllowed,                  //3
   CommandSerialGetCloneAllowed,                   //4
   CommandSerialGetMaxAmountUsers,                 //5
   CommandSerialGetDeleteUserAllowed,              //6
   CommandSerialGetDeleteLastUserAllowed,          //7
   CommandSerialGetRemoteUpd,                      //8
   CommandSerialGetKeyImport,                      //9
   CommandSerialGetPKCS11,                         //10
   CommandSerialGetMSCNG,                          //11
   CommandSerialGetJCE,                            //12
   CommandSerialGetAllowSecLogRemote,              //13
   CommandSerialGetAllowSecLogClient,              //14
   CommandSerialGetSyslogLevel,                    //15
   CommandSerialGetSNMP,                           //16
   CommandSerialGetCloneKeyCreate,                 //17
   CommandSerialGetRemoteCloning,                  //18
   CommandSerialGetKeyExport,                      //19
   CommandSerialGetMaxSoCards,                     //20
   CommandSerialGetCloneSecConfigAllowed,          //21
   CommandNoLongerUsedGetFips,                     //22
   CommandSerialGetRestoreAllowed,                 //23
   CommandSerialGetAllowAddGenesis,                //24
   CommandSerialGetUserDecanus_Obsolete,           //25
   CommandSerialGetUserConfig,                     //26
   CommandNoLongerUsedGetRestrictedMode,           //27
   CommandSerialGetUserSO_Obsolete,                //28
   CommandSerialGetMasterUrl,                      //29
   CommandSerialGetPartitionHA_Obsolete,           //30
   CommandSerialGetUserDiskLimit,                  //31
   CommandSerialGetCryptologLevel,                 //32
   CommandSerialGetAllowVirtualCards,              //33
   CommandSerialGetUserSetupPasswordTimeout,       //34
   CommandSerialGetNoLongerReadablePkcs11Password, //35
   CommandSerialGetPartitionReadOnly,              //36
   CommandSerialGetKeyInvalidation,                //37
   CommandSerialGetVerifyBlockState,               //38
   CommandSerialGetHsmNoiseSource,                 //39
   CommandSerialGetUserManagement,                 //40
   CommandSerialGetUserCrypto,                     //41
   CommandSerialGetAllowSysLogRemote,              //42
   CommandSerialGetSoAccess,                       //43
   CommandSerialGetKeyExtract,                     //44
   CommandSerialGetExternalStorage,                //45
   WriteConfigDeviceCommand = 0x3300,
   CommandSerialImportNetworkConfig,               //1
   CommandSerialSetSystemTime,                     //2
   CommandSerialSetDate_unsued,                    //3
   CommandSerialSetDeviceName,                     //4
   CommandSerialSetDNSIPv4,                        //5
   CommandSerialSetDNSIPv6,                        //6
   CommandSerialSetPort,                           //7
   CommandNoLongerUsedSetIPv6Port,                 //8
   CommandSerialSetDHCPIPv4,                       //9
   CommandSerialSetIPv6,                           //10
   CommandSerialSetIPv4,                           //11
   CommandSerialSetNmaskIPv4,                      //12
   CommandSerialSetGWIPv4,                         //13
   CommandSerialSetSyslogIP,                       //14
   CommandSerialSetCryptologIP,                    //15
   CommandSerialSetFsmState,                       //17
   CommandSerialSetSinglePort,                     //18
   CommandSerialSetNtpUrl,                         //18
   CommandSerialSetNtpKey,                         //19
   CommandSerialSetSyslogKey,                      //20
   CommandSerialSetCryptologKey,                   //21
   CommandSerialSetRedundancy,                     //22
   CommandSerialSetSyslogSize,                     //23
   CommandSerialSetCryptologSize,                  //24
   CommandSerialSetSyslogPort,                     //25
   CommandSerialSetCryptologPort,                  //26
   CommandSerialSetNmaskIPv6,                      //27
   CommandSerialSetGWIPv6,                         //28
   CommandSerialSetDNSDomain,                      //29
   CommandSerialSetDHCPIPv6,                       //30
   CommandSerialSetStaticNetworkConfig,            //31
   CommandSerialSetStaticIpv4RouteConfig,          //32
   CommandSerialSetStaticIpv6RouteConfig,          //33
   CommandSerialPauseSync,                         //34
   CommandSerialResumeSync,                        //35
   CommandSerialSetDecanus,                        //36
   CommandSerialSetTiltSensor,                     //37
   CommandSerialPauseClientInterfaces,             //38
   CommandSerialResumeClientInterfaces,            //39
   CommandSerialSetSyslogTcp,                      //40
   CommandSerialSetCryptologTcp,                   //41
   CommandSerialSetSelftestFailure,                //42
   CommandSerialSetAutoNeg,                        //43
   CommandSerialSetLineSpeed,                      //44
   CommandSerialSetFullDuplex,                     //45
   CommandSerialOverrideMasterUrl,                 //46
   CommandSerialSetProtcolInterface,               //47
   CommandSerialSetSnmpInterface,                  //48
   CommandSerialSetNtpInterface,                   //49
   CommandSerialSetSyslogInterface,                //50
   CommandSerialSetCryptologInterface,             //51
   WriteConfigSecurityCommand = 0x3400,
   CommandSerialImportSoConfig,                    //1
   CommandSerialSetNtp,                            //2
   CommandSerialSetBackupAllowed,                  //3
   CommandSerialSetCloneAllowed,                   //4
   CommandSerialSetMaxAmountUsers,                 //5
   CommandSerialSetDeleteUserAllowed,              //6
   CommandSerialSetDeleteLastUserAllowed,          //7
   CommandSerialSetRemoteUpd,                      //8
   CommandSerialSetAllowSecLogRemote,              //9
   CommandSerialSetAllowSecLogClient,              //10
   CommandSerialSetSyslogLevel,                    //11
   CommandSerialSetSNMP,                           //12
   CommandSerialSetRemoteCloning,                  //13
   CommandSerialSetCloneSecConfigAllowed,          //14
   CommandNoLongerUsedSetFips,                     //15
   CommandSerialSetRestoreAllowed,                 //16
   CommandSerialSetAllowAddGenesis,                //17
   CommandNoLongerUsedSetRestrictedMode,           //18
   CommandSerialSetMasterUrl,                      //19
   CommandSerialSetAccessPassword,                 //20
   CommandSerialRenameUserConfig,                  //21
   CommandSerialCreateUserConfig,                  //22
   CommandSerialSetCryptologLevel,                 //23
   CommandSerialSetHsmNoiseSource,                 //24
   CommandSerialSetAllowSysLogRemote,              //25
   WriteUserConfigSecurityCommand = 0x3500,
   CommandSerialSetKeyImport,                      //1
   CommandSerialSetKeyExport,                      //2
   CommandSerialSetPKCS11,                         //3
   CommandSerialSetMSCNG,                          //4
   CommandSerialSetJCE,                            //5
   CommandSerialSetUserDecanus_Obsolete,           //6
   CommandSerialSetUserConfig,                     //7
   CommandSerialSetMaxSoCards,                     //8
   CommandSerialSetPartitionHA_Obsolete,           //9
   CommandSerialSetUserDiskLimit,                  //10
   CommandSerialSetCloneKeyCreate,                 //11
   CommandSerialSetUserSO_Obsolete,                //12
   CommandSerialSetUserSetupPasswordTimeout,       //13
   CommandSerialSetPkcsPassword,                   //14
   CommandSerialSetPartitionReadOnly,              //15
   CommandSerialSetKeyInvalidation,                //16
   CommandSerialSetVerifyBlockState,               //17
   CommandSerialSetUserManagement,                 //18
   CommandSerialSetUserCrypto,                     //19
   CommandSerialSetSoAccess,                       //20
   CommandSerialImportUserConfig,                  //21
   CommandSerialSetKeyExtract,                     //22
   CommandSerialSetExternalStorage,                //23
   DmuCommand = 0x4000,
   CommandSerialError,                             //1
   CommandSerialRomParams,                         //2
   CommandSerialRomChunk,                          //3
   CommandSerialUpdateSuccess,                     //4
   CommandSerialHalt,                              //5
   CommandSerialInitFwUpdate,                      //6
   CommandSerialFwUpdateBlock,                     //7
   CommandSerialFwUpdateComplete,                  //8
   CommandSerialStartTest1,                        //9
   CommandSerialWriteTestLog,                      //10
   CommandSerialPublishSerialNumber,               //11
   CommandSerialGetCurrentIp,                      //12
   CommandSerialEnterAccessPassword,               //13
   CommandSerialTamperEvent,                       //14
   CommandSerialCheckUserExists,                   //15
   CommandSerialSetCurrentConfigUser,              //16
   CommandSerialSetSingleUserConfigUser_Obsolete,  //17
   CommandSerialGetCurrentConfigUser,              //18
   CommandSerialWriteFirmwareIcv,                  //19
   CommandSerialAnnounceCryptoMode,                //20
   CommandSerialQueryMessages,                     //21 CommandSerialFsmRequestQuery
   CommandSerialNoMessages,                        //22 CommandSerialFsmRequestNone
   CommandSerialCheckFrimwareIcv,                  //23 CommandSerialFsmRequestDoMarvAuth
   CommandSerialFactoryReset,                      //24 CommandSerialFsmRequestDoFactoryReset
   CommandSerialCreateKek,                         //25 CommandSerialFsmRequestDoCreateKek
   CommandSerialNotifyError,                       //26 CommandSerialFsmRequestDoSendErrorSignal
   CommandSerialStartGenesisTimeoutWatchdog_Obsolete, //27 CommandSerialFsmRequestDoStartGenesisTimeoutWatchdog
   CommandSerialReadKek,                           //28 CommandSerialFsmRequestDoSendKeKtoMarv
   CommandSerialGenerateEntropy,                   //29 CommandSerialFsmRequestDoSendRandomNr
   CommandSerialGenerateNewSeal,                   //30 CommandSerialFsmRequestDoCreateNewSeal
   CommandSerialAnnounceReady,                     //31 CommandSerialFsmRequestDoAnnounceReady
   CommandSerialLinkStateNotification,             //32 CommandSerialFsmRequestDoNotifyLinkState
   CommandSerialGetDipSwitchState,                 //33 CommandSerialFsmRequestDoGetDipSwitch
   CommandSerialGetRtcTime,                        //34 CommandSerialFsmRequestDoGetTime
   CommandSerialSetRtcTime,                        //35 CommandSerialFsmRequestDoSyncTime
   CommandSerialGetCurrentNumberOfSo,              //36 CommandSerialFsmRequestDoGetAmountCurrentShares
   CommandSerialReadAlertLogs,                     //37 CommandSerialFsmRequestDoReadAlertLogs
   CommandSerialReadDeviceSoData,                  //38 CommandSerialFsmRequestDoReadDeviceSoData
   CommandSerialWriteDeviceSoData,                 //39 CommandSerialFsmRequestDoWriteDeviceSoData
   CommandSerialCryptoStateNotification,           //40 CommandSerialFsmRequestDoNotifyCryptoState
   CommandSerialLoggedInConnectionNotification,    //41 CommandSerialFsmRequestDoNotifyLoggedInConnectionCount
   CommandSerialAcknowledgeAlerts,                 //42 CommandSerialFsmRequestDoAckAlerts
   CommandSerialReadMasterSoData,                  //43 CommandSerialFsmRequestDoReadMasterSoData
   CommandSerialWriteMasterSoData,                 //44 CommandSerialFsmRequestDoWriteMasterSoData
   CommandSerialStartUpdate,                       //45 CommandSerialFsmRequestDoStartDmuUpdate
   CommandSerialReadCek,                           //46 CommandSerialFsmRequestDoSendCEKtoMarv
   CommandSerialCheckTamper,                       //47 CommandSerialFsmRequestDoCheckTamper
   CommandSerialInsertVirtualCard,                 //48 CommandSerialFsmRequestDoInsertVirtualCard
   CommandSerialEjectVirtualCard,                  //49 CommandSerialFsmRequestDoEjectVirtualCard
   CommandSerialReadDigitalSeal,                   //50 CommandSerialFsmRequestDoSendDigSeal
   CommandSerialExecuteAuthenticationChange,       //51 CommandSerialFsmRequestDoExecuteCardChange
   CommandSerialPrepareCardChange,                 //52 CommandSerialFsmRequestDoPrepareCardChange
   CommandSerialFinishCardChange,                  //53 CommandSerialFsmRequestDoFinishCardChange
   CommandSerialGetDiagnostics,                    //54 CommandSerialFsmRequestDoGetDiagnostics
   CommandSerialClearSoRecord,                     //55 CommandSerialFsmRequestDoCreateSO
   CommandSerialDeleteSo,                          //56 CommandSerialFsmRequestDoDeleteSO
   CommandSerialWriteSoCard,                       //57 CommandSerialFsmRequestDoWriteSOShareCard
   CommandSerialCheckGenesis,                      //58 CommandSerialFsmRequestDoCheckGenesis
   CommandSerialCheckSoRole,                       //59 CommandSerialFsmRequestDoCheckSORole
   CommandSerialCheckMasterSoRole,                 //60 CommandSerialFsmRequestDoCheckMasterSORole
   CommandSerialCheckPin,                          //61 CommandSerialFsmRequestDoCheckPIN
   CommandSerialCheckMasterPin,                    //62 CommandSerialFsmRequestDoCheckMasterPIN
   CommandSerialGetCardType,                       //63 CommandSerialFsmRequestDoGetCardType
   CommandSerialWritePinToCard,                    //64 CommandSerialFsmRequestDoWriteSOPinToCard
   CommandSerialCheckRestorePin,                   //65 CommandSerialFsmRequestDoCheckRestorePIN
   CommandSerialCopyGenesisCard,                   //66 CommandSerialFsmRequestDoCopyGenesisCard
   CommandSerialPasteGenesisCard,                  //67 CommandSerialFsmRequestDoPasteGenesisCard
   CommandSerialReadRestorePin,                    //68 CommandSerialFsmRequestDoReadRestorePinFromGenesis
   CommandSerialAnnounceSignatureType,             //69 CommandSerialFsmRequestDoAnnounceSignatureType
   CommandSerialCheckDrbg,                         //70
   CommandSerialSetCardPeer,                       //71
   CommandSerialSetAuthMode,                       //72
   CommandSerialGetNofPinAttempts,                 //73
   CommandSerialHaStateNotification,               //74
   CommandSerialDecanusStateNotification,          //75
   CommandSerialGetDmuVersionLimit,                //76
   CommandSerialChangeTiltSensorState,             //77
   CommandSerialCreateGnPinDisclosureSecret,       //78
   CommandSerialReadGenesisPinRecord,              //79
   CommandSerialWriteGenesisCard,                  //80
   CommandSerialGenerateRandom,                    //81 CTR-DRBG
   CommandSerialGenerateNoise,                     //82 Raw noise before conditioning
   CommandSerialRestartEntropy,                    //83
   CommandSerialSendSelftestFailure,               //84
   CommandSerialRequestSelftestFailure,            //85
   CommandSerialUpdateGenesisCard,                 //86
   CommandSerialCheckLockedGenesisCardAllowed,     //87
   CommandSerialCopySoToMasterSo_Obsolete,         //88
   CommandSerialCheckDegradedMaster,               //89
   CommandSerialClearMasterSOs,                    //90
   CommandSerialSoStateNotification,               //91
   CommandSerialChangeHsmNoiseSource,              //92
   CommandSerialSecureModuleKeys,                  //93
   CommandSerialPublishPublicSerialNumber,         //94
   CommandSerialGetTemperatures,                   //95
   CommandSerialGetFanspeeds,                      //96
   CommandSerialCreatePik,                         //97
   CommandSerialSignPikCertificate,                //98
   CommandSerialSignWithPik,                       //99
   CommandSerialUpdatePublicSerialNumber,         //100
   CommandSerialResetRks,                         //101
   CommandSerialClearRks,                         //102
   CommandSerialWriteMgmtBackupCard,              //103
   CommandSerialGetMgmtBackupCardData,            //104
   CommandSerialResetCardCounter,                 //105
   CommandSerialDecreaseCardCounter,              //106
   DmuInternalCommand = 0x5000,
   CommandDmuGetTime,                              //1
   CommandDmuGetDate,                              //2
   CommandDmuSetTime,                              //3
   CommandDmuSetDate,                              //4

   CommandIdMask = 0x00FF,
   CommandTypeMask = 0xF000,
   CommandMask = 0xFF00,
   CommandInvalid = 0xFFFF,
};

enum Data_type : uint16_t {
   dataTypeUnknown = 0x0000,
   // 32-bit type IDs
   typeGeneric32Bit = 0x0001,
   typeKeySize = 0x0002,
   typeKeyCount = 0x0003,
   typeCryptoMethod = 0x0004,
   typeObjectType = 0x0005,
   typeKeyType = 0x0006,
   typeAlgorithmType = 0x0007,
   typeError = 0x0008,
   typeTcpCookie = 0x0009,
   typeMagic = 0x000a,
   typeCtrGcmSize = 0x000b,
   typeCreationTime = 0x000c,
   typeModificationTime = 0x000d,
   typeTcpPort = 0x000e,
   typeBool = 0x000f,
   typeTestCode = 0x0010,
   typeCTime = 0x0011,
   typeConfigBoolean = 0x0012,
   typeCommandId = 0x0013,
   typeUserCount = 0x0014,
   typeSensorCount = 0x0015,
   typeSystemTemperatur = 0x0016,
   typeFanSpeed = 0x0017,
   typeInterfaceNumber = 0x0018,
   typeDipSwitchState = 0x0019,
   typeFsmState = 0x001a,
   typeKeyAccessFlag = 0x001b,
   typeKeyCapabilityFlag = 0x001c,
   typeAlertCount = 0x001d,
   typeEcCurveId = 0x001e,
   typeSoSharesCount = 0x001f,
   typeCardSlot = 0x0020,
   typeDisplayMsg = 0x0021,
   typeSignal = 0x0022,
   typeCardType = 0x0023,
   typeSequenceIndex = 0x0024,
   typeIpv4Address = 0x0025,
   typeUserAction = 0x0026,
   typeRequestId = 0x0027,
   typeConnectionStatus = 0x0028,
   typeBytesToProcess = 0x0029,
   typeBytesToAllocate = 0x002a,
   typeAcceleratorCount = 0x002b,
   typeDiagnosticsId = 0x002c,
   typePasswordValidTime = 0x002d,
   typeTimeInMs = 0x002e,
   typeKeyFieldType = 0x002f,
   typeKeyFreefield = 0x0030,
   typeUserSize = 0x0031,
   typeCryptoMode = 0x0032,
   typeFileSize = 0x0033,
   typeFileCount = 0x0034,
   typeSnoozeTime = 0x0035,
   typeCardDataAddress = 0x0036,
   typeCardDataLength = 0x0037,
   typeDecanusCount = 0x0038,
   typeTamperState = 0x0039,
   typeSignatureCount = 0x003A,
   typeEkaOperation = 0x003B,
   typeApprovalCount = 0x003C,
   typeCounter = 0x003D,
   typePrimeSize = 0x003E,
   typeHashSize = 0x003F,
   typeUrlCount = 0x0040,
   typeNodeCount = 0x0041,
   typeUsageRate = 0x0042,
   typeMemorySize = 0x0043,
   typeProtocolCount = 0x0044,
   typeProtocol = 0x0045,
   typeModuleCount = 0x0046,
   typeServerCount = 0x0047,
   typeRouteCount = 0x0048,
   typeSensorId = 0x0049,
   typeFanId = 0x004A,
   typeSlotCount = 0x004B,
   typeAlertId = 0x004C,
   typeInterfaceCount = 0x004D,
   typeOperatorId = 0x004E,
   typeHardwareRevision = 0x004F,
   typeLedStatus = 0x0050,
   typeDateDay = 0x0051,
   typeDateMonth = 0x0052,
   typeDateYear = 0x0053,
   typeTimeHour = 0x0054,
   typeTimeMinute = 0x0055,
   typeTimeSecond = 0x0056,
   typePinAttempts = 0x0057,
   typeSubPrimeSize = 0x0058,
   typeTokenCount = 0x0059,
   typeGroupCount = 0x005A,
   typeSignaturesRequired = 0x005B,
   typeKeyFreefieldJce = 0x005C,
   typeKeyFreefieldPkcs = 0x005D,
   typeKeyFreefieldCng = 0x005E,
   typeCryptoCurrency = 0x005F,
   typeKeySecurity = 0x0060,
   typeKeyIndex = 0x0061,
   typeWindowSize = 0x0062,
   typeHsmNoiseSource = 0x0063,
   typePublicSerialNumber = 0x0064,
   typeMgmtConfigType = 0x0065,
   typeCertificateCount = 0x0066,
   typeMgmtShareCoordinates = 0x0067,
   typeShareCount = 0x0068,
   // 64-bit type IDs
   typeGeneric64Bit = 0x0101,
   typeSerialNumber = 0x0102,
   typeAlertMessage = 0x0103,
   typeDeviceUid = 0x0104,
   typeStartDate = 0x0105,
   typeEndDate = 0x0106,
   typeSecondsSinceEpoch = 0x0107,
   typeSizeInBytes = 0x0108,
   typeObjectCount64 = 0x0109,
   type64BitCounter = 0x0110,
   // Misc. type IDs
   typeGenericString = 0x1001,
   typeIdLabel = 0x1002,
   typeLoginData = 0x1003,
   typeSignData = 0x1004,
   typePlainData = 0x1005,
   typeEncryptedData = 0x1006,
   typeIvData = 0x1007,
   typeKeyData = 0x1008,
   typeCertificateData = 0x1009,
   typeKeyPass = 0x100a,
   typeDeviceName = 0x100b,
   typeWrappedKey = 0x100c,
   typeVerifyData = 0x100d,
   typeSignatureData = 0x100e,
   typeModulus = 0x100f,
   typePublicExponent = 0x1010,
   typePrivateExponent = 0x1011,
   typePrime = 0x1012,
   typeCoefficient = 0x1013,
   typeSubPrime = 0x1014,
   typeBase = 0x1015,
   typePublicData = 0x1016,
   typeRandomSize = 0x1017,
   typeRandomData = 0x1018,
   typeRandomSeed = 0x1019,
   typeAadData = 0x101a,
   typePSourceData = 0x101b,
   typeDigestData = 0x101c,
   typeHashData = 0x101d,
   typeDeviceVersion = 0x101e,
   typeSoPassword = 0x101f,
   typeUserId = 0x1020,
   typeConfigString = 0x1021,
   typeLogMessage = 0x1022,
   typeUserPassword = 0x1023,
   typeNoLongerValid = 0x1024,
   typeSoData = 0x1025,
   typeChecksum = 0x1026,
   typeEcCurveData = 0x1027,
   typeEcCurvePoint = 0x1028,
   typeSharedData = 0x1029,
   typeOrder = 0x102a,
   typeCofactor = 0x102b,
   typeIpv6Address = 0x102c,
   typeUrl = 0x102d,
   typeUserSerial = 0x102e,
   typeUserData = 0x102f,
   typeClusterData = 0x1030,
   typeObjectId = 0x1031,
   typeObjectLabel = 0x1032,
   typeObjectData = 0x1033,
   typeFileName = 0x1034,
   typeSignKey = 0x1035,
   typeEncryptionKey = 0x1036,
   typeCardPin = 0x1037,
   typeDiagnosticsString = 0x1038,
   typeRestorePassword = 0x1039,
   typeRestorePin = 0x103a,
   typeKeyFieldValue = 0x103b,
   typeFileData = 0x103c,
   typeCardData = 0x103d,
   typeLicensedModule = 0x103e,
   typeDomain = 0x103f,
   typeDnsServer = 0x1040,
   typeNtpServer = 0x1041,
   typeSyslogServer = 0x1042,
   typeSeclogServer = 0x1043,
   typeMacAddress = 0x1044,
   typeRouteDestination = 0x1045,
   typeRouteGateway = 0x1046,
   typeAcceleratorName = 0x1047,
   typeCpuInfo = 0x1048,
   typeSystemType = 0x1049,
   typeIssuer = 0x104A,
   typeSubject = 0x104B,
   typeApplication = 0x104C,
   typeConfigInterfaceName = 0x104D,
   typeUsageBlob = 0x104E,
   typeBlockBlob = 0x104F,
   typeUnblockBlob = 0x1050,
   typeModifyBlob = 0x1051,
   typePublicKeyEncoded = 0x1052,
   typeEkaKeyData = 0x1053,
   typeEkaTimeStamp = 0x1054,
   typeApprovalToken = 0x1055,
   typeDerSignature = 0x1056,
   typeEkaUsagePayload = 0x1057,
   typeEkaModifyPayload = 0x1058,
   typeDisclosureSecret = 0x1059,
   typeEkaModifyString = 0x105A,
   typeCryptoCurrencyAddress = 0x105B,
   typeKeySeed = 0x105C,
   typeChainCode = 0x105D,
   typePkcs8Container = 0x105E,
   typePkcs8Password = 0x105F,
   typeXmlKeyData = 0x1060,
   typeAuthorizationInfo = 0x1061,
   typeTimestampRequest = 0x1062,
   typeTimestampResponse = 0x1063,
   typeMgmtShareParameter = 0x1064,
   typeExternalKey = 0x1065,
   typeCounterId = 0x1066,
   typeGeneralizedTime = 0x1067,
   // Skipped during deserialization. Do not use for sane data, is intended for communication obfuscation
   typeGarbageData = 0xDEAD,
};

	struct __attribute__ ((packed)) Header
	{
		uint32_t protocol_version;
		uint32_t data_length;
		uint32_t command_id;
		uint32_t session_id;
		uint32_t network_transaction_number;
		uint32_t hsm_data_generation_number;
		uint32_t status;
		uint32_t reserved0;
		uint32_t reserved1;
		uint32_t reserved2;
	};

	struct __attribute__ ((packed)) Payload_header
	{
		uint16_t type;
		uint16_t length;
		uint32_t long_length;
	};

	enum {
		PROTOCOL_VERSION = 0x00050000,
		LOGIN_MAGIC = 0x73636573
	};

uint8_t magic_login_request[32] = { 0x69, 0xa5, 0x2d, 0x3f,
                                    0x56, 0x56, 0x63, 0xd1,
                                    0xea, 0x14, 0xc7, 0xa1,
                                    0xe3, 0xe1, 0x21, 0xc4,
                                    0xb9, 0xf9, 0x20, 0x3b,
                                    0x71, 0x68, 0xa2, 0x6c,
                                    0xb1, 0x4a, 0x4f, 0x8d,
                                    0xdd, 0x80, 0x5e, 0x83 };

uint8_t magic_login_auth[32]    = { 0x7c, 0xc8, 0x60, 0x19,
                                    0xfa, 0x2f, 0xd4, 0xd5,
                                    0xc8, 0x83, 0x24, 0x0c,
                                    0xea, 0x44, 0x61, 0x80,
                                    0x04, 0xbb, 0x0c, 0xc7,
                                    0x34, 0x65, 0x74, 0xb5,
                                    0x70, 0xce, 0x45, 0xc6,
                                    0xcf, 0x71, 0x29, 0x63 };

}
